#!/usr/bin/env node

// src/bin/index.ts
import { parseArgs } from "util";

// src/bin/commands/init.ts
import childProcess from "child_process";
import fs from "fs/promises";
import json5 from "json5";
import ora from "ora";
import path from "path";
import util from "util";
var exec = util.promisify(childProcess.exec);
var exists = async (file) => {
  try {
    await fs.access(path.join(process.cwd(), file));
  } catch (err) {
    if (err.code === "ENOENT") {
      return false;
    } else {
      throw err;
    }
  }
  return true;
};
var init_default = async (positionals2) => {
  const spinner = ora("Initializing project").start();
  const lastPositional = positionals2[positionals2.length - 1];
  const spaceHandle = lastPositional === "init" ? null : lastPositional;
  if (!spaceHandle) {
    spinner.fail("Please provide a space handle like this:\n$ ronin init my-space");
    process.exit(1);
  }
  if (!await exists("package.json")) {
    spinner.fail("No `package.json` found in the current directory. Please run the command in your project.");
    process.exit(1);
  }
  const packageManager = await exists("bun.lockb") ? "bun" : "npm";
  const packageManagerName = packageManager === "bun" ? "Bun" : "npm";
  spinner.text = `Detected ${packageManagerName} \u2014 installing types package`;
  try {
    if (packageManager === "bun") {
      await exec(`bun add @ronin/${spaceHandle} --dev`);
    } else {
      await exec(`npm install @ronin/${spaceHandle} --save-dev`);
    }
    const tsConfigPath = path.join(process.cwd(), "tsconfig.json");
    const tsConfigExists = await exists("tsconfig.json");
    if (!tsConfigExists) return;
    const contents = await fs.readFile(tsConfigPath, "utf-8");
    const tsConfig = json5.parse(contents);
    if (!tsConfig.compilerOptions.types || !tsConfig.compilerOptions.types.includes(`@ronin/${spaceHandle}`))
      Object.assign(tsConfig.compilerOptions, {
        types: [...tsConfig.compilerOptions.types || [], `@ronin/${spaceHandle}`]
      });
    await fs.writeFile(tsConfigPath, JSON.stringify(tsConfig, null, 2));
  } catch (err) {
    if (err instanceof Error && err.message.includes("401")) {
      spinner.fail(`You are not a member of the "${spaceHandle}" space or the space doesn't exist.`);
      process.exit(1);
    }
  }
  spinner.succeed("Project initialized");
};

// src/bin/commands/login.ts
import getPort from "get-port";
import http from "http";
import open from "open";
import ora2 from "ora";

// src/bin/utils/session.ts
import toml from "@iarna/toml";
import fs2 from "fs/promises";
import ini from "ini";
import os from "os";
import path2 from "path";
var CACHE_DIR = path2.join(os.homedir(), ".ronin");
var CACHE_DIR_FILE = path2.join(CACHE_DIR, "session.json");
var getSession = async () => {
  try {
    const contents = await fs2.readFile(CACHE_DIR_FILE, {
      encoding: "utf-8"
    });
    const parsedContents = JSON.parse(contents);
    if (parsedContents?.token) return parsedContents;
  } catch (err) {
    if (err.code !== "ENOENT") throw err;
  }
  return null;
};
var readConfigFile = async (filePath, name, parser) => {
  let npmConfigContents = {};
  try {
    const contents = await fs2.readFile(filePath, { encoding: "utf-8" });
    npmConfigContents = parser(contents);
  } catch (err) {
    if (err.code !== "ENOENT") {
      console.error(`Failed to read ${name} config file at ${filePath}`, err);
    }
  }
  return npmConfigContents;
};
var writeConfigFile = async (filePath, contents) => {
  return fs2.writeFile(filePath, contents, { encoding: "utf-8" });
};
var storeSession = async (token) => {
  try {
    await fs2.stat(CACHE_DIR);
  } catch (err) {
    if (err.code === "ENOENT") {
      await fs2.mkdir(CACHE_DIR);
    } else {
      throw err;
    }
  }
  return writeConfigFile(CACHE_DIR_FILE, JSON.stringify({ token }, null, 2));
};
var storeTokenForNPM = async (token) => {
  const npmConfigFile = process.env["npm_config_userconfig"] || path2.join(os.homedir(), ".npmrc");
  const npmConfigContents = await readConfigFile(npmConfigFile, "npm", ini.parse);
  npmConfigContents["@ronin:registry"] = "https://ronin.supply";
  npmConfigContents["//ronin.supply/:_authToken"] = token;
  await writeConfigFile(npmConfigFile, ini.stringify(npmConfigContents));
};
var storeTokenForBun = async (token) => {
  const bunConfigFile = path2.join(os.homedir(), ".bunfig.toml");
  const bunConfigContents = await readConfigFile(bunConfigFile, "Bun", toml.parse);
  if (!bunConfigContents.install) bunConfigContents.install = {};
  if (!bunConfigContents.install.scopes) bunConfigContents.install.scopes = {};
  if (!bunConfigContents.install.scopes.ronin) bunConfigContents.install.scopes.ronin = {};
  bunConfigContents.install.scopes.ronin.url = "https://ronin.supply";
  bunConfigContents.install.scopes.ronin.token = token;
  await writeConfigFile(bunConfigFile, toml.stringify(bunConfigContents));
};

// src/bin/commands/login.ts
var login_default = async (appToken) => {
  const spinner = ora2("Logging in").start();
  if (appToken) {
    await Promise.all([storeTokenForNPM(appToken), storeTokenForBun(appToken)]);
    spinner.succeed("Logged in successfully!");
    return;
  }
  let sessionToken;
  const port = await getPort();
  const server = http.createServer().listen(port);
  const baseURL = new URL("https://ronin.co/actions/auth/cli");
  const currentHost = `http://localhost:${port}`;
  const initURL = new URL(baseURL);
  initURL.searchParams.set("from", currentHost);
  spinner.text = `Please log in using the following URL:
${initURL.href}`;
  spinner.suffixText = `
`;
  try {
    [sessionToken] = await Promise.all([
      new Promise((resolve, reject) => {
        server.once("request", (req, res) => {
          res.setHeader("connection", "close");
          const requestURL = new URL(req.url || "", currentHost);
          const requestToken = requestURL.searchParams.get("token");
          if (!requestToken) return reject(new Error("Missing token"));
          resolve(requestToken);
          const headers = { Location: baseURL.href };
          res.writeHead(307, headers).end();
        });
        server.once("error", reject);
      }),
      open(initURL.href)
    ]);
  } finally {
    server.close();
  }
  await Promise.all([
    storeSession(sessionToken),
    storeTokenForNPM(sessionToken),
    storeTokenForBun(sessionToken)
  ]);
  spinner.suffixText = "";
  spinner.succeed("Logged in successfully!");
};

// src/bin/utils/info.ts
import chalkTemplate from "chalk-template";

// package.json
var version = "4.0.14";

// src/bin/utils/info.ts
var printVersion = () => {
  console.log(version);
  process.exit(0);
};
var printHelp = () => {
  const text = chalkTemplate`
  {bold.magenta ronin} — Data at the edge

  {bold USAGE}

      {bold $} {bold.magenta ronin}
      {bold $} {bold.magenta ronin} login
      {bold $} {bold.magenta ronin} --help
      {bold $} {bold.magenta ronin} --version

  {bold COMMANDS}

      login                               Authenticate with RONIN (run by default for every command)
      init [space]                        Initialize the TypeScript types for a given space

  {bold OPTIONS}

      -h, --help                          Shows this help message
      -v, --version                       Shows the version of the CLI that is currently installed
      -d, --debug                         Shows additional debugging information
  `;
  console.log(text);
  process.exit(0);
};

// src/bin/index.ts
var values;
var positionals;
try {
  ({ values, positionals } = parseArgs({
    args: process.argv,
    options: {
      help: {
        type: "boolean",
        short: "h",
        default: false
      },
      version: {
        type: "boolean",
        short: "v",
        default: false
      },
      debug: {
        type: "boolean",
        short: "d",
        default: false
      }
    },
    strict: true,
    allowPositionals: true
  }));
} catch (err) {
  if (err instanceof Error) {
    console.error(err.message);
  } else {
    throw err;
  }
  process.exit(1);
}
var run = async () => {
  if (values.help) return printHelp();
  if (values.version) return printVersion();
  const normalizedPositionals = positionals.map((positional) => positional.toLowerCase());
  const appToken = process.env.RONIN_TOKEN;
  const session = await getSession();
  if (!process.stdout.isTTY && !session && !appToken) {
    let message = "If RONIN CLI is invoked from a non-interactive shell, ";
    message += "a `RONIN_TOKEN` environment variable containing an app token must be provided.";
    console.error(message);
    process.exit(1);
  }
  if (!session && !normalizedPositionals.includes("login")) await login_default(appToken);
  if (normalizedPositionals.includes("login")) return login_default(appToken);
  if (normalizedPositionals.includes("init")) return init_default(positionals);
  return printHelp();
};
run();
